@@ -17,6 +17,7 @@
 Network-related utilities and helper functions.
 """
 
+import ipaddress
 import logging
 import os
 import re
@@ -24,8 +25,9 @@ import socket
 from urllib import parse
 
 import netaddr
+from netaddr.core import INET_ATON
 from netaddr.core import INET_PTON
-import netifaces
+import psutil
 
 from oslo_utils._i18n import _
 
@@ -99,8 +101,11 @@ def is_valid_ipv4(address, strict=None):
        (``a.b.c.d``) as opposed to address format
        (``a.b.c.d``, ``a.b.c``, ``a.b``, ``a``).
     """
+    if not address:
+        return False
+
     if strict is not None:
-        flag = INET_PTON if strict else 0
+        flag = INET_PTON if strict else INET_ATON
         try:
             return netaddr.valid_ipv4(address, flags=flag)
         except netaddr.AddrFormatError:
@@ -111,7 +116,7 @@ def is_valid_ipv4(address, strict=None):
         if netaddr.valid_ipv4(address, flags=INET_PTON):
             return True
         else:
-            if netaddr.valid_ipv4(address):
+            if netaddr.valid_ipv4(address, flags=INET_ATON):
                 LOG.warning(
                     'Converting in non strict mode is deprecated. '
                     'You should pass strict=False if you want to '
@@ -147,6 +152,30 @@ def is_valid_ipv6(address):
         return False
 
 
+def get_noscope_ipv6(address):
+    """Take an IPv6 address and trim scope out if present.
+
+    :param address: Value to change
+    :type address: string
+    :returns: string
+
+    .. versionadded: 7.3.0:
+    """
+    # TODO(egarciar): Scope was added for IPv6Address on Python 3.9. Same for
+    # the removesuffix str method. Remove try/except when minimum required
+    # version for Python is 3.9
+    try:
+        _ipv6 = ipaddress.IPv6Address(address)
+        if _ipv6.scope_id:
+            address = address.removesuffix('%' + _ipv6.scope_id)
+        return address
+    except (ipaddress.AddressValueError, AttributeError):
+        if is_valid_ipv6(address):
+            parts = address.rsplit("%", 1)
+            return parts[0]
+        raise
+
+
 def is_valid_cidr(address):
     """Verify that address represents a valid CIDR address.
 
@@ -202,6 +231,9 @@ def get_ipv6_addr_by_EUI64(prefix, mac):
 
     .. versionadded:: 1.4
     """
+    if not isinstance(prefix, str):
+        msg = _("Prefix must be a string")
+        raise TypeError(msg)
     # Check if the prefix is an IPv4 address
     if is_valid_ipv4(prefix):
         msg = _("Unable to generate IP address by EUI64 for IPv4 prefix")
@@ -380,11 +412,9 @@ def get_my_ipv4():
        Return ``'127.0.0.1'`` if there is no default interface.
     """
     try:
-        csock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        csock.connect(('192.0.2.0', 80))
-        (addr, port) = csock.getsockname()
-        csock.close()
-        return addr
+        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as csock:
+            csock.connect(('192.0.2.0', 80))
+            return csock.getsockname()[0]
     except socket.error:
         return _get_my_ipv4_address()
 
@@ -393,16 +423,23 @@ def _get_my_ipv4_address():
     """Figure out the best ipv4
     """
     LOCALHOST = '127.0.0.1'
-    gtw = netifaces.gateways()
-    try:
-        interface = gtw['default'][netifaces.AF_INET][1]
-    except (KeyError, IndexError):
-        LOG.info('Could not determine default network interface, '
-                 'using 127.0.0.1 for IPv4 address')
-        return LOCALHOST
+    interface = None
+
+    with open('/proc/net/route') as routes:
+        for route in routes:
+            route_attrs = route.strip().split('\t')
+            if route_attrs[1] == '00000000':
+                interface = route_attrs[0]
+                break
+        else:
+            LOG.info('Could not determine default network interface, '
+                     'using %s for IPv4 address', LOCALHOST)
+            return LOCALHOST
 
     try:
-        return netifaces.ifaddresses(interface)[netifaces.AF_INET][0]['addr']
+        addrs = psutil.net_if_addrs()[interface]
+        v4addrs = [addr for addr in addrs if addr.family == socket.AF_INET]
+        return v4addrs[0].address
     except (KeyError, IndexError):
         LOG.info('Could not determine IPv4 address for interface %s, '
                  'using 127.0.0.1',
@@ -426,11 +463,9 @@ def get_my_ipv6():
        Return ``'::1'`` if there is no default interface.
     """
     try:
-        csock = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
-        csock.connect(('2001:db8::1', 80))
-        (addr, _, _, _) = csock.getsockname()
-        csock.close()
-        return addr
+        with socket.socket(socket.AF_INET6, socket.SOCK_DGRAM) as csock:
+            csock.connect(('2001:db8::1', 80))
+            return csock.getsockname()[0]
     except socket.error:
         return _get_my_ipv6_address()
 
@@ -439,16 +474,25 @@ def _get_my_ipv6_address():
     """Figure out the best IPv6 address
     """
     LOCALHOST = '::1'
-    gtw = netifaces.gateways()
-    try:
-        interface = gtw['default'][netifaces.AF_INET6][1]
-    except (KeyError, IndexError):
-        LOG.info('Could not determine default network interface, '
-                 'using %s for IPv6 address', LOCALHOST)
-        return LOCALHOST
+    interface = None
+    ZERO_ADDRESS = '00000000000000000000000000000000'
+
+    with open('/proc/net/ipv6_route') as routes:
+        for route in routes:
+            route_attrs = route.strip().split(' ')
+            if ((route_attrs[0], route_attrs[1]) == (ZERO_ADDRESS, '00') and
+                    (route_attrs[2], route_attrs[3]) == (ZERO_ADDRESS, '00')):
+                interface = route_attrs[-1]
+                break
+        else:
+            LOG.info('Could not determine default network interface, '
+                     'using %s for IPv6 address', LOCALHOST)
+            return LOCALHOST
 
     try:
-        return netifaces.ifaddresses(interface)[netifaces.AF_INET6][0]['addr']
+        addrs = psutil.net_if_addrs()[interface]
+        v6addrs = [addr for addr in addrs if addr.family == socket.AF_INET6]
+        return v6addrs[0].address
     except (KeyError, IndexError):
         LOG.info('Could not determine IPv6 address for interface '
                  '%(interface)s, using %(address)s',
